@@ -1,7 +1,7 @@
 //! Working with pages of a PDF document.
 
 use std::cell::OnceCell;
-use std::num::NonZeroUsize;
+use std::num::NonZeroU32;
 use std::ops::DerefMut;
 
 use pdf_writer::types::TabOrder;
@@ -42,14 +42,21 @@ pub struct PageSettings {
 
 impl PageSettings {
     /// Create new page settings and define the size of the page surface.
-    pub fn new(width: f32, height: f32) -> Self {
+    pub fn new(size: Size) -> Self {
         Self {
-            media_box: Some(Rect::from_xywh(0.0, 0.0, width, height).unwrap()),
-            surface_size: Size::from_wh(width, height).unwrap(),
+            media_box: Some(Rect::from_xywh(0.0, 0.0, size.width(), size.height()).unwrap()),
+            surface_size: size,
             ..Default::default()
         }
     }
 
+    /// Try to create new page settings with the specified width and height.
+    ///
+    /// Returns `None` if either the width or the height is not > 0.
+    pub fn from_wh(width: f32, height: f32) -> Option<Self> {
+        Some(Self::new(Size::from_wh(width, height)?))
+    }
+
     /// Change the media box.
     ///
     /// The media box defines the visible area of the page when opening the PDF,
@@ -461,7 +468,7 @@ pub struct PageLabel {
     /// The prefix of the page label.
     pub(crate) prefix: Option<String>,
     /// The numeric value of the page label.
-    pub(crate) offset: Option<NonZeroUsize>,
+    pub(crate) offset: Option<NonZeroU32>,
 }
 
 impl PageLabel {
@@ -469,7 +476,7 @@ impl PageLabel {
     pub fn new(
         style: Option<NumberingStyle>,
         prefix: Option<String>,
-        offset: Option<NonZeroUsize>,
+        offset: Option<NonZeroU32>,
     ) -> Self {
         Self {
             style,
