@@ -5,6 +5,7 @@ use std::ops::{Deref, DerefMut};
 use std::rc::Rc;
 use std::sync::Arc;
 
+use indexmap::IndexMap;
 use pdf_writer::types::{StructRole, StructRole2};
 use pdf_writer::writers::{OutputIntent, StructTreeRoot};
 use pdf_writer::{Chunk, Finish, Limits, Name, Pdf, Ref, Str, TextStr};
@@ -29,7 +30,7 @@ use crate::resource;
 use crate::resource::{Resource, Resourceable};
 use crate::surface::{Location, Surface};
 use crate::text::GlyphId;
-use crate::text::{Font, FontContainer, FontIdentifier, FontInfo};
+use crate::text::{Font, FontContainer, FontIdentifier};
 use crate::util::{Deferred, SipHashable};
 
 /// Settings that should be applied when creating a PDF document.
@@ -206,10 +207,6 @@ pub(crate) enum MaybeDeviceColorSpace {
 /// - Annotations used in the document.
 ///   etc.
 pub(crate) struct SerializeContext {
-    /// A cache for mapping `FontInfo`s to existing Font objects. Is mainly used to
-    /// speed up SVG conversion, so that if we convert many SVGs with the same font,
-    /// we can cache the font.
-    pub(crate) font_cache: HashMap<Arc<FontInfo>, Font>,
     /// The ref of the page tree.
     page_tree_ref: Ref,
     /// PDF 2.0 namespaces.
@@ -257,7 +254,6 @@ impl SerializeContext {
 
         Self {
             cached_mappings: HashMap::new(),
-            font_cache: HashMap::new(),
             pdf2_ns,
             global_objects: GlobalObjects::default(),
             cur_ref,
@@ -376,12 +372,7 @@ impl SerializeContext {
         self.global_objects
             .font_map
             .entry(font.clone())
-            .or_insert_with(|| {
-                self.font_cache
-                    .insert(font.font_info().clone(), font.clone());
-
-                Rc::new(RefCell::new(FontContainer::new(font.clone())))
-            })
+            .or_insert_with(|| Rc::new(RefCell::new(FontContainer::new(font.clone()))))
             .clone()
     }
 
@@ -655,9 +646,7 @@ impl SerializeContext {
 
     fn serialize_fonts(&mut self) -> KrillaResult<()> {
         let fonts = self.global_objects.font_map.take();
-        let mut sorted = fonts.values().collect::<Vec<_>>();
-        sorted.sort_by_key(|e| e.borrow().font().sip_hash());
-        for font_container in sorted {
+        for font_container in fonts.values() {
             let borrowed = font_container.borrow();
 
             if !borrowed.type3_mapper().is_empty() {
@@ -938,7 +927,7 @@ pub(crate) struct GlobalObjects {
     // Needs to be pub(crate) because writing of named destinations happens in `ChunkContainer`.
     pub(crate) named_destinations: MaybeTaken<HashMap<NamedDestination, Ref>>,
     /// A map from fonts to font container.
-    font_map: MaybeTaken<HashMap<Font, Rc<RefCell<FontContainer>>>>,
+    font_map: MaybeTaken<IndexMap<Font, Rc<RefCell<FontContainer>>>>,
     /// All XYZ destinations used in the document. The reason we need to store them
     /// separately is that we can only serialize them in the very end, once all pages
     /// have been written, so that we know the Ref of the page they belong to.
